#ifdef GL_ES
precision mediump float;
#endif

varying vec2 v_texcoord;

#ifdef GL_ES
uniform highp vec2 u_uvOffset;
#else
uniform vec2 u_uvOffset;
#endif

#ifdef BLEND_TEX2
varying vec2 v_texcoord1;
#endif

#ifdef BLEND_TEX3
varying vec2 v_texcoord2;
#endif

#ifdef WITH_VCOLOR
varying vec4 v_vColor;
#endif

uniform vec3 u_AmbientLightSourceColor;
uniform vec4 u_color;

const int MAX_DIRECTIONAL_LIGHT_NUM = 1;
uniform vec3 u_DirLightSourceColor[MAX_DIRECTIONAL_LIGHT_NUM];
uniform vec3 u_DirLightSourceDirection[MAX_DIRECTIONAL_LIGHT_NUM];

uniform vec3 u_fogColor;
varying float v_fogFactor;

//
vec4 texOver(vec4 c0, vec4 c1, float a)
{
    return vec4( mix((c0.rgb * c0.a) + (c1.rgb * c1.a), c1.rgb, c1.a), min(1.0, c0.a + c1.a) );
}

vec4 texModulate(vec4 c0, vec4 c1, float a)
{
    a *= c1.a;
    return vec4( mix(c0.rgb, c0.rgb * c1.rgb, a), c0.a );
}

vec4 texAdditive(vec4 c0, vec4 c1, float a)
{
    a *= c1.a;
    return vec4( c0.rgb + c1.rgb * a, c0.a );
}

vec4 texSubtract(vec4 c0, vec4 c1, float a)
{
    a *= c1.a;
    return vec4( c0.rgb - c1.rgb * a, c0.a );
}

vec4 texLuminosity(vec4 c0, vec4 c1, float a)
{// Modulate2x系 として利用
    a *= c1.a;
    return vec4( c0.rgb * c1.rgb * a, c0.a );
}

//
void main()
{
    u_AmbientLightSourceColor;
    u_DirLightSourceColor[0];
    u_DirLightSourceDirection[0];
    
    vec4 c = texture2D(CC_Texture0, v_texcoord + u_uvOffset);
    
#ifdef BLEND_TEX2
    c = BLEND_TEX2(c, texture2D(CC_Texture1, v_texcoord1 + u_uvOffset), 1.0);
#endif
    
#ifdef BLEND_TEX3
    c = BLEND_TEX3(c, texture2D(CC_Texture2, v_texcoord2 + u_uvOffset), 1.0);
#endif
    
#ifdef WITH_VCOLOR
    c = c * v_vColor;
#endif
    
    
#if FOG_MODE == 2
    gl_FragColor = vec4( c.rgb + u_fogColor * v_fogFactor, c.a ) * u_color;
#elif FOG_MODE == 1
    gl_FragColor = vec4( mix(c.rgb, u_fogColor, v_fogFactor), c.a ) * u_color;
#else
    u_fogColor, v_fogFactor;
    gl_FragColor = c * u_color;
#endif
}
